// src/builders/Button.ts
var isURLButton = (button) => {
  if (button === void 0)
    return false;
  return button.url !== void 0;
};
var isCustomIdButton = (button) => {
  if (button === void 0)
    return false;
  return button.custom_id !== void 0;
};
var Button = class {
  label;
  style;
  custom_id;
  url;
  emoji;
  disabled;
  constructor(data) {
    if (data?.label) {
      this.label = data.label;
    }
    if (data?.style) {
      this.style = data.style;
    }
    if (isURLButton(data)) {
      this.url = data.url;
    }
    if (isCustomIdButton(data)) {
      this.custom_id = data.custom_id;
    }
    if (data?.emoji) {
      this.emoji = data.emoji;
    }
    if (data?.disabled) {
      this.disabled = data.disabled;
    }
  }
  setLabel(label) {
    this.label = label;
    return this;
  }
  setStyle(style) {
    this.style = style;
    return this;
  }
  setCustomId(custom_id) {
    this.custom_id = custom_id;
    return this;
  }
  setURL(url) {
    this.url = url;
    return this;
  }
  setEmoji(emoji) {
    if (typeof emoji === "string") {
      this.emoji = {
        name: emoji
      };
    } else {
      this.emoji = emoji;
    }
    return this;
  }
  setDisabled(disabled) {
    this.disabled = disabled;
    return this;
  }
  toJSON() {
    return {
      type: 2,
      label: this.label,
      style: this.style || 1,
      custom_id: this.custom_id || "",
      url: this.url,
      emoji: this.emoji,
      disabled: this.disabled
    };
  }
};

// src/builders/Embed.ts
var HEXCODE_REGEX = /^#?([a-fA-F0-9]{6})$/;
var URL_REGEX = /(https?:\/\/(?:www\.|(?!www))[a-zA-Z0-9][a-zA-Z0-9-]+[a-zA-Z0-9]\.[^\s]{2,}|www\.[a-zA-Z0-9][a-zA-Z0-9-]+[a-zA-Z0-9]\.[^\s]{2,}|https?:\/\/(?:www\.|(?!www))[a-zA-Z0-9]+\.[^\s]{2,}|www\.[a-zA-Z0-9]+\.[^\s]{2,})/gi;
var Embed = class {
  title;
  description;
  url;
  timestamp;
  color;
  footer;
  image;
  thumbnail;
  author;
  fields;
  constructor(data) {
    if (data?.title) {
      this.title = data.title;
    }
    if (data?.description) {
      this.description = data.description;
    }
    if (data?.url) {
      this.url = data.url;
    }
    if (data?.timestamp) {
      this.timestamp = data.timestamp;
    }
    if (data?.color) {
      this.color = data.color;
    }
    if (data?.footer) {
      this.footer = data.footer;
    }
    if (data?.image) {
      this.image = data.image;
    }
    if (data?.thumbnail) {
      this.thumbnail = data.thumbnail;
    }
    if (data?.author) {
      this.author = data.author;
    }
    this.fields = data?.fields || [];
  }
  setTitle(title) {
    if (typeof title !== "string") {
      throw new TypeError("Title must be of type string");
    }
    if (title.length > 256) {
      throw new TypeError("Title should not exceed 256 characters");
    }
    this.title = title;
    return this;
  }
  setDescription(description) {
    if (typeof description !== "string") {
      throw new TypeError("Description must be of type string");
    }
    if (description.length > 4096) {
      throw new TypeError("Description should not exceed 4096 characters");
    }
    this.description = description;
    return this;
  }
  setColor(color) {
    if (typeof color !== "string" && typeof color !== "number") {
      throw new TypeError(
        `Invalid color type, Expected string / number instead received ${typeof color}`
      );
    }
    if (typeof color === "number") {
      if (color < 0 || color > 16777215) {
        throw new TypeError("Invalid color");
      }
      this.color = color;
    } else {
      const colorvalidation = color.match(HEXCODE_REGEX);
      if (!colorvalidation) {
        throw new TypeError("Invalid HexCode!");
      }
      this.color = parseInt(colorvalidation[1], 16);
    }
    return this;
  }
  setURL(url) {
    if (typeof url !== "string") {
      throw new TypeError("Url should be of type string");
    }
    if (!URL_REGEX.test(url)) {
      throw new TypeError("Not a valid URL");
    }
    this.url = url;
    return this;
  }
  setTimestamp(timestamp = new Date()) {
    if (Number.isNaN(new Date(timestamp).getTime())) {
      throw new TypeError("Invalid Timestamp date.");
    }
    this.timestamp = new Date(timestamp);
    return this;
  }
  setFooter(iconURL, text) {
    if (typeof text !== "string") {
      throw new TypeError("Footer text should be of type string");
    }
    if (text.length > 2048) {
      throw new TypeError(
        "Embed footer text should not be more than 2048 characters"
      );
    }
    this.footer = { text };
    if (iconURL !== void 0) {
      if (typeof iconURL !== "string") {
        throw new TypeError("Url should be of type string");
      }
      if (!iconURL.startsWith("attachment://") && !URL_REGEX.test(iconURL)) {
        throw new TypeError("Not a valid URL");
      }
      this.footer.icon_url = iconURL;
    }
    return this;
  }
  setImage(imageURL) {
    if (typeof imageURL !== "string") {
      throw new TypeError("Image url should be of type string");
    }
    if (!imageURL.startsWith("attachment://") && !URL_REGEX.test(imageURL)) {
      throw new TypeError("Not a valid URL");
    }
    this.image = { url: imageURL };
    return this;
  }
  setThumbnail(url) {
    if (typeof url !== "string") {
      throw new TypeError("Thumbnail url should be of type string");
    }
    this.thumbnail = { url };
    return this;
  }
  setAuthor(name, url, iconURL) {
    if (typeof name !== "string") {
      throw new TypeError("Footer text should be of type string");
    }
    if (name.length > 256) {
      throw new TypeError("Footer text should not exceed 256 characters");
    }
    this.author = { name };
    return this;
  }
  addField(name, value, inline) {
    if (this.fields.length >= 25) {
      throw new TypeError("Embeds cannot exceed 25 fields");
    }
    if (typeof name !== "string") {
      throw new TypeError("Embed field name should be of type string");
    }
    if (typeof value !== "string") {
      throw new TypeError("Embed field value should be of type string");
    }
    if (typeof inline !== "boolean") {
      throw new TypeError("Embed field inline should be of type boolean");
    }
    if (name.length > 256) {
      throw new TypeError("Embed field name cannot exceed 256 characters");
    }
    if (value.length > 1024) {
      throw new TypeError("Embed field value cannot exceed 1024 characters");
    }
    this.fields.push({ name, value, inline });
    return this;
  }
  addFields(fields) {
    for (const field of fields) {
      this.addField(field.name, field.value, field.inline);
    }
    return this;
  }
  toJSON() {
    return {
      title: this.title,
      description: this.description,
      url: this.url,
      color: this.color,
      timestamp: this.timestamp?.toString(),
      footer: this.footer,
      image: this.image,
      thumbnail: this.thumbnail
    };
  }
};

// src/MessageBuilder.ts
var MessageBuilder = class {
  data;
  constructor(content) {
    this.data = {
      embeds: [],
      components: [],
      content
    };
  }
  addEmbed(embed) {
    if (embed instanceof Embed) {
      this.data.embeds.push(embed.toJSON());
    } else {
      this.data.embeds.push(embed);
    }
    return this;
  }
  removeEmbed(index) {
    this.data.embeds.splice(index, 1);
    return this;
  }
  clearEmbeds() {
    this.data.embeds = [];
    return this;
  }
  addComponent(component) {
    if (component instanceof Button) {
      this.data.components.push(component.toJSON());
    } else {
      this.data.components.push(component);
    }
    return this;
  }
  toJSON() {
    return this.data;
  }
};

// src/Webhook.ts
import phin from "phin";
import FormData from "form-data";
var Webhook = class {
  constructor(options) {
    this.options = options;
  }
  defaultPayload;
  async get() {
    const data = await this.request(this.options.url);
    return data;
  }
  setUsername(username) {
    this.defaultPayload = {
      ...this.defaultPayload,
      username
    };
  }
  setAvatar(avatar) {
    this.defaultPayload = {
      ...this.defaultPayload,
      avatar_url: avatar
    };
  }
  async modify(options) {
    return this.request(this.options.url, "PATCH", options);
  }
  async delete() {
    return this.request(this.options.url, "DELETE");
  }
  async execute(data) {
    return this.defaultPayload ? this.request(
      this.options.url,
      "POST",
      {
        ...this.defaultPayload,
        ...data
      },
      data.files
    ) : this.request(this.options.url, "POST", data, data.files);
  }
  async request(url, method, postData, file) {
    if (!method)
      method = "GET";
    if (!postData)
      postData = null;
    if (typeof postData === "object")
      postData = JSON.stringify(postData);
    return new Promise((resolve, reject) => {
      if (file) {
        const form = new FormData();
        if (Array.isArray(file)) {
          for (const f of file) {
            form.append(f.name, f.file, f.name);
          }
        } else {
          form.append(file.name, file.file, file.name);
        }
        form.append("payload_json", postData);
        form.submit(url, (err, res) => {
          if (err)
            reject(err);
          res.on("data", (d) => {
            resolve(JSON.parse(d.toString()));
          });
        });
      } else {
        phin({
          url,
          method,
          data: postData,
          headers: {
            "Content-Type": "application/json"
          },
          parse: "json"
        }).then((res) => {
          console.log(res.body);
          resolve(res.body);
        }).catch(reject);
      }
    });
  }
};
export {
  Button,
  Embed,
  MessageBuilder,
  Webhook
};
